@@ -233,6 +233,28 @@ class ConfigManager(object):
 
       return ConfigManager._GetNumber(uio, prompt, previousValue, minValue=minValue, maxValue=maxValue, numberType=ConfigManager.FLOAT_NUMBER_TYPE)
 
+    @staticmethod
+    def GetBool(uio, prompt, previousValue=True):
+        """@brief Input a boolean value.
+           @param uio          A UIO (User Input Output) instance.
+           @param prompt       The prompt presented to the user in order to enter
+                                the float value.
+           @param previousValue The previous True or False value."""
+        _prompt = "%s y/n" % (prompt)
+
+        if previousValue:
+            prevValue='y'
+        else:
+            prevValue='n'
+
+        while True:
+            value = ConfigManager.GetString(uio, _prompt, prevValue)
+            value=value.lower()
+            if value == 'n':
+                return False
+            elif value == 'y':
+                return True
+
     @staticmethod
     def GetPrivateKeyFile():
         """@brief Get the private key file."""
@@ -284,6 +306,41 @@ class ConfigManager(object):
         decryptedBytes = crypter.decrypt(token)
         return decryptedBytes.decode('utf-8')
 
+    @staticmethod
+    def GetConfigFile(cfgFilename, addDotToFilename=True, cfgPath=None):
+        """@brief Get the config file."""
+
+        if not cfgFilename:
+            raise Exception("No config filename defined.")
+
+        cfgFilename = cfgFilename
+        if addDotToFilename:
+            if not cfgFilename.startswith("."):
+
+                cfgFilename=".%s" % (cfgFilename)
+
+            else:
+
+                cfgFilename=cfgFilename
+
+        #The the config path has been set then use it
+        if cfgPath:
+            configPath = cfgPath
+
+        else:
+            configPath=""
+            #If an absolute path is set for the config file then don't try to
+            #put the file in the users home dir
+            if not cfgFilename.startswith("/"):
+                configPath = expanduser("~")
+                configPath = configPath.strip()
+                #If no user is known then default to root user.
+                #This occurs on Omega2 startup apps.
+                if len(configPath) == 0 or configPath == '/' or configPath == '/root':
+                    configPath="/root"
+
+        return join( configPath, cfgFilename )
+
     def __init__(self, uio, cfgFilename, defaultConfig, addDotToFilename=True, encrypt=False, cfgPath=None):
         """@brief Constructor
            @param uio A UIO (User Input Output) instance. May be set to None if no user messages are required.
@@ -295,7 +352,7 @@ class ConfigManager(object):
                           This is not secure but assuming the private key has not been compromised it's
                           probably the best we can do. Therefore if encrypt is set True then the
                           an ssh key must be present in the ~/.ssh folder named id_rsa.
-           @param cfgPath The config path when the config file will be stored. By default this is unset and the 
+           @param cfgPath The config path when the config file will be stored. By default this is unset and the
                           current users home folder is the location of the config file."""
         self._uio               = uio
         self._cfgFilename       = cfgFilename
@@ -307,7 +364,7 @@ class ConfigManager(object):
 
         self._cfgFile = self._getConfigFile()
         self._modifiedTime = self._getModifiedTime()
-        
+
     def _info(self, msg):
         """@brief Display an info message if we have a UIO instance.
            @param msg The message to be displayed."""
@@ -322,37 +379,7 @@ class ConfigManager(object):
 
     def _getConfigFile(self):
         """@brief Get the config file."""
-
-        if not self._cfgFilename:
-            raise Exception("No config filename defined.")
-
-        cfgFilename = self._cfgFilename
-        if self._addDotToFilename:
-            if not self._cfgFilename.startswith("."):
-
-                cfgFilename=".%s" % (self._cfgFilename)
-
-            else:
-
-                cfgFilename=self._cfgFilename
-
-        #The the config path has been set then use it
-        if self._cfgPath:
-            configPath = self._cfgPath
-
-        else:
-            configPath=""
-            #If an absolute path is set for the config file then don't try to 
-            #put the file in the users home dir
-            if not self._cfgFilename.startswith("/"):
-                configPath = expanduser("~")
-                configPath = configPath.strip()
-                #If no user is known then default to root user.
-                #This occurs on Omega2 startup apps.
-                if len(configPath) == 0 or configPath == '/' or configPath == '/root':
-                    configPath="/root"
-
-        return join( configPath, cfgFilename )
+        return ConfigManager.GetConfigFile(self._cfgFilename, addDotToFilename=self._addDotToFilename, cfgPath=self._cfgPath)
 
     def addAttr(self, key, value):
         """@brief Add an attribute value to the config.
@@ -535,9 +562,9 @@ class ConfigManager(object):
         previousValue=self._getValue(key)
         yes = self.getYesNo(prompt, previousValue=previousValue)
         if yes:
-            value=1
+            value=True
         else:
-            value=0
+            value=False
         self.addAttr(key, value)
 
     def getYesNo(self, prompt, previousValue=0):
@@ -672,7 +699,7 @@ class ConfigManager(object):
         running=True
         while running:
             idKeyDict=self.show()
-            response = self._uio.getInput("Enter 'E' to edit a parameter, 'S' to save and quit or 'Q' to quit")
+            response = self._uio.getInput("Enter 'E' to edit a parameter, or 'Q' to quit")
             response=response.upper()
             if response == 'E':
                 id = self._uio.getIntInput("Enter the ID of the parameter to change")
@@ -681,9 +708,8 @@ class ConfigManager(object):
                 else:
                     key=idKeyDict[id]
                     editConfigMethod(key)
-            elif response == 'S':
-                self.store()
-                running = False
+                    self.store()
+
             elif response == 'Q':
                 running = False
 
