@@ -2,7 +2,7 @@
 
 import  os
 import  socket
-from    paramiko import SSHClient, AutoAddPolicy, RSAKey, AuthenticationException, SFTPClient
+from    paramiko import SSHClient, AutoAddPolicy, AuthenticationException, SFTPClient
 import  logging
 import  threading
 import  socketserver
@@ -48,7 +48,7 @@ class ExtendedSSHClient(SSHClient):
         super(ExtendedSSHClient, self).__init__()
         self.set_missing_host_key_policy(AutoAddPolicy())
 
-    def __exec_command(self, command, bufsize=-1):
+    def __exec_command(self, command, bufsize=-1, timeout=None):
         """
         Execute a command on the SSH server.  A new L{Channel} is opened and
         the requested command is executed.  The command's input and output
@@ -59,12 +59,15 @@ class ExtendedSSHClient(SSHClient):
         @type command: str
         @param bufsize: interpreted the same way as by the built-in C{file()} function in python
         @type bufsize: int
+        @param timeout The timeout value in seconds or None if no command timeout.
+        @param type None or float
         @return: the stdin, stdout, and stderr of the executing command
         @rtype: tuple(L{ChannelFile}, L{ChannelFile}, L{ChannelFile})
 
         @raise SSHException: if the server fails to execute the command
         """
-        chan = self._transport.open_session()
+        chan = self._transport.open_session(timeout=timeout)
+        chan.settimeout(timeout)
         chan.exec_command(command)
         stdin = chan.makefile('wb', bufsize)
         stdout = chan.makefile('rb', bufsize)
@@ -86,22 +89,49 @@ class ExtendedSSHClient(SSHClient):
         chan.exec_command(command)
         return chan
 
-    def runCmd(self, cmd, throwError=True):
-        """Run a command over an ssh session and return a tuple with the following threee elements
-          0 - the return code/exit status of the command
-          1 - Lines of text from stdout
-          2 - lines of text from stderr
-
-          If throwError is true and the exit status of the copmmand is not 0 then an
-          SSHError will be thrown
+    def runCmd(self, cmd, throwError=True, timeout=None):
+        """@brief Run a command over an ssh session
+           @param cmd The command to execute.
+                      Note that if the command is executed in the background (I.E the command ends with a '
+                      character then no attempt to read from stdout or stderr is made.
+                      In this case the stdout and stderr lines returned will be empty lists but
+                      the return code will still be valid (I.E 0 if the backgrounded command started).
+           @param throwError If True and the command fails throw an error. If False and the command fail re
+                             If throwError is True and the exit status of the command is not 0 then an SSHE
+           @param timeout The timeout value in seconds or None (default) if no command timeout is required.
+           @return A tuple with the following three elements if the command executes
+                        0 - the return code/exit status of the command
+                        1 - Lines of text from stdout
+                        2 - lines of text from stderr
+                   If the command fails to execute then None is returned.
         """
-        stdin, stdout, stderr, exitStatus = self.__exec_command(cmd)
-        if throwError and exitStatus != 0:
-            errorText = stderr.read()
-            if len(errorText) > 0:
-                raise SSHError(errorText)
-            raise SSHError("The cmd '%s' return the error code: %d" % (cmd, exitStatus))
-        return [exitStatus, ExtendedSSHClient.GetLines(stdout.read()), ExtendedSSHClient.GetLines(stderr.read())]
+        readData=True
+        stdOutLines=[]
+        stdErrLines=[]
+        try:
+            # Check if we have a backgrounded command. If so don't read the output
+            # as openssh servers may block reading from stdout on backgrounded commands
+            # which is not what the caller would expect.
+            sCmd = cmd.strip()
+            if sCmd.endswith("&"):
+                readData=False
+            stdin, stdout, stderr, exitStatus = self.__exec_command(cmd,timeout=timeout)
+            if throwError and exitStatus != 0:
+                if readData:
+                    errorText = stderr.read(1024)
+                    if len(errorText) > 0:
+                        raise SSHError(errorText)
+                raise SSHError("The cmd '%s' return the error code: %d" % (cmd, exitStatus))
+            if readData:
+                stdOutLines = ExtendedSSHClient.GetLines(stdout.read())
+                stdErrLines = ExtendedSSHClient.GetLines(stderr.read())
+            return [exitStatus, stdOutLines, stdErrLines]
+        except:
+            if throwError:
+                raise
+        return None
+
+
 
 class SSH(object):
     """@brief responsible for connecting an ssh connection, excuting commands."""
@@ -116,6 +146,19 @@ class SSH(object):
     SERVER_AUTHORISED_KEYS_FILE         = "~/.ssh/authorized_keys"
     DEFAULT_SSH_CONNECTION_TIMEOUT      = 20
 
+    @staticmethod
+    def AddKey(privateKey):
+        """@brief Add a key to the list of keys that will be tried when connecting to an ssh server.
+                  This method should be called before instantiating the SSH class. If a key is
+                  already present it won't be added to avoid duplicates.
+           @param privateKey The name of the private key file in the ~/.ssh folder. The public
+                             key file must have the same name with a .pub extension."""
+        if privateKey not in SSH.PRIVATE_KEY_FILE_LIST:
+            SSH.PRIVATE_KEY_FILE_LIST.insert(0, privateKey)
+
+        if privateKey not in SSH.PUBLIC_KEY_FILE_LIST:
+            SSH.PUBLIC_KEY_FILE_LIST.insert(0, privateKey+".pub")
+
     @staticmethod
     def GetPublicKeyFile():
         """@brief Get the public key file from the <HOME>/.ssh"""
@@ -134,6 +177,26 @@ class SSH(object):
 
         raise SSHError("Unable to find a public key file. Please use the 'ssh-keygen -t rsa' command to generate a key pair.")
 
+    @staticmethod
+    def GetPublicKeyFileList():
+        """@brief Get a list of public key files that exist locally in <HOME>/.ssh"""
+        homeFolder = SSH.LOCAL_SSH_CONFIG_PATH
+        if not os.path.isdir(homeFolder):
+            username = getuser()
+            if username == 'root':
+                homeFolder = '/root/.ssh'
+            else:
+                homeFolder = '/home/%s/.ssh' % (username)
+
+        keyFileList = []
+        for key in SSH.PUBLIC_KEY_FILE_LIST:
+            keyFile = os.path.join(homeFolder, key)
+            if os.path.isfile(keyFile):
+                if keyFile not in keyFileList:
+                    keyFileList.append(keyFile)
+
+        return keyFileList
+
     @staticmethod
     def GetPrivateKeyFile():
         """@brief Get the private key file from the <HOME>/.ssh"""
@@ -152,6 +215,25 @@ class SSH(object):
 
         raise SSHError("Unable to find a public key file. Please use the 'ssh-keygen -t rsa' command to generate a key pair.")
 
+    @staticmethod
+    def GetPrivateKeyFileList():
+        """@brief Get a list of private key files that exist locally in <HOME>/.ssh"""
+        homeFolder = SSH.LOCAL_SSH_CONFIG_PATH
+        if not os.path.isdir(homeFolder):
+            username = getuser()
+            if username == 'root':
+                homeFolder = '/root/.ssh'
+            else:
+                homeFolder = '/home/%s/.ssh' % (username)
+
+        keyFileList = []
+        for key in SSH.PRIVATE_KEY_FILE_LIST:
+            keyFile = os.path.join(homeFolder, key)
+            if os.path.isfile(keyFile):
+                if keyFile not in keyFileList:
+                    keyFileList.append(keyFile)
+        return keyFileList
+
     @staticmethod
     def GetPublicKey():
         """@brief Get the public ssh key from the local machine
@@ -219,11 +301,7 @@ class SSH(object):
         self._password          = password
         self._uio               = uio
         if privateKeyFile:
-            self._privateKeyFile    = privateKeyFile
-        else:
-            self._privateKeyFile = SSH.GetPrivateKeyFile()
-
-        self._sshPrivateKey = RSAKey.from_private_key_file(self._privateKeyFile)
+            SSH.AddKey(privateKeyFile)
 
         self._ssh = ExtendedSSHClient()
         logging.getLogger("paramiko").setLevel(logging.WARNING)
@@ -242,6 +320,12 @@ class SSH(object):
         if self._uio:
             self._uio.warn(text)
 
+    def _debug(self, text):
+        """@brief Present debug level message to the user.
+           @param text The text to be presented to the user."""
+        if self._uio:
+            self._uio.debug(text)
+
     def _connect(self, connectSFTPSession=False, timeout=DEFAULT_SSH_CONNECTION_TIMEOUT):
         """@brief Connect the ssh connection
            @param connectSFTPSession If True then just after the ssh connection
@@ -251,8 +335,35 @@ class SSH(object):
         if not self._ssh:
             self._ssh = ExtendedSSHClient()
 
-        self._ssh.connect(self._host, username=self._username, password=self._password, port=self._port,
-                          pkey=self._sshPrivateKey, timeout=timeout)
+        connected = False
+        for privateKeyFile in SSH.GetPrivateKeyFileList():
+            if os.path.isfile(privateKeyFile):
+                msg = "Trying private key {}".format(privateKeyFile)
+                self._debug(msg)
+                try:
+                    #Define the ssh config
+                    cfg = {
+                        'hostname': self._host,
+                        'port': self._port,
+                        'timeout': timeout,
+                        'username': self._username,
+                        'key_filename': privateKeyFile,
+                        # This is required of else loging in without the password fails.
+                        'disabled_algorithms': dict(pubkeys=['rsa-sha2-256', 'rsa-sha2-512'])
+                    }
+                    # If we have a password then add this to the config
+                    if self._password and len(self._password) > 0:
+                        cfg['password']=self._password
+                    self._ssh.connect(**cfg)
+                    connected = True
+                    break
+
+                except:
+                    pass
+
+        if not connected:
+            raise Exception("Failed to connect to the SSH server {}@{}.".format(self._username, self._host))
+
         # It can be usefull to know what local IP address was used to reach the ssh server
         self._localAddress = self._ssh.get_transport().sock.getsockname()[0]
         self._ssh.get_transport().use_compression(self.useCompression)
@@ -283,17 +394,18 @@ class SSH(object):
            @return The ssh transport object."""
         return self._ssh.get_transport()
 
-    def runCmd(self, cmd, throwError = True):
-        """@brief Run a command over an ssh session.
-           @return A tuple with the following three elements when the command is complete.
-          0 - the return code/exit status of the command
-          1 - Lines of text from stdout
-          2 - lines of text from stderr
-
-          If throwError is true and the exit status of the copmmand is not 0 then an
-          SSHError will be thrown
+    def runCmd(self, cmd, throwError=True, timeout=None):
+        """@brief Run a command over an ssh session
+           @param cmd The command to execute
+           @param throwError If True and the command fails throw an error. If False and the command fail return the erorr code etc.
+                             If throwError is True and the exit status of the command is not 0 then an SSHError will be thrown
+           @param timeout The timeout value in seconds or None (default) if no command timeout is required.
+           @return A tuple with the following threee elements
+              0 - the return code/exit status of the command
+              1 - Lines of text from stdout
+              2 - lines of text from stderr
         """
-        return self._ssh.runCmd(cmd, throwError=throwError)
+        return self._ssh.runCmd(cmd, throwError=throwError, timeout=timeout)
 
     def startCmd(self, cmd):
         """@brief Start executing a command. This will return after starting the command and before the command has completed.
@@ -332,14 +444,6 @@ class SSH(object):
     def _setupAutologin(self, timeout=DEFAULT_SSH_CONNECTION_TIMEOUT):
         """@brief Setup autologin on the ssh server.
            @param timeout The connection timeout in seconds."""
-
-        publicKeyFilename = "{}.pub".format(self._privateKeyFile)
-        publicKeyFile = os.path.join(SSH.LOCAL_SSH_CONFIG_PATH, publicKeyFilename)
-        if not os.path.isfile(publicKeyFile):
-            raise SSHError(
-                "{} public key file not found. Please create a public/private key pair and try again.".format(
-                    publicKeyFile))
-
         self._warn("Auto login to the ssh server failed authentication.")
         self._info("Copying the local public ssh key to the ssh server for automatic login.")
         self._info("Please enter the ssh server ({}) password for the user: {}".format(self._host, self._username))
@@ -536,7 +640,7 @@ class SSHTunnelManager(object):
     """@brief Responsible for setting up, tearing down and maintaining lists of
               SSH port forwarding and ssh reverse port forwarding connections."""
 
-    RX_BUFFER_SIZE = 4096
+    RX_BUFFER_SIZE = 65535
 
     def __init__(self, uio, ssh, useCompression):
         """@brief Constructor
@@ -552,6 +656,24 @@ class SSHTunnelManager(object):
         self._forwardingServerList = []
         self._reverseSShDict = {}
 
+    def _info(self, text):
+        """@brief Present an info level message to the user.
+           @param text The text to be presented to the user."""
+        if self._uio:
+            self._uio.info(text)
+
+    def _warn(self, text):
+        """@brief Present an warning level message to the user.
+           @param text The text to be presented to the user."""
+        if self._uio:
+            self._uio.warn(text)
+
+    def _error(self, text):
+        """@brief Present an error level message to the user.
+           @param text The text to be presented to the user."""
+        if self._uio:
+            self._uio.error(text)
+
     def startFwdSSHTunnel(self, serverPort, destHost, destPort):
         """@brief Start an ssh port forwarding tunnel. This is a non blocking method.
                   A separate thread will be started to handle data transfer over the
@@ -563,7 +685,7 @@ class SSHTunnelManager(object):
                              end of the ssh connection.
            @param destPort   The host TCP port of the tunnel destination at the remote
                              end of the ssh connection."""
-        self._uio.info("Forwarding local TCP server port (%d) to %s:%d on the remote end of the ssh connection." % (
+        self._info("Forwarding local TCP server port (%d) to %s:%d on the remote end of the ssh connection." % (
         serverPort, destHost, destPort))
         transport = self._ssh.getTransport()
 
@@ -589,14 +711,14 @@ class SSHTunnelManager(object):
             if forwardingServerPort == serverPort:
                 forwardingServer.shutdown()
                 forwardingServer.server_close()
-                self._uio.info("Shutdown ssh port forwarding connection using local server port %d." % (serverPort))
+                self._info("Shutdown ssh port forwarding connection using local server port %d." % (serverPort))
 
     def stopAllFwdSSHTunnels(self):
         """@brief Stop all previously started ssh port forwarding servers.."""
         for forwardingServer in self._forwardingServerList:
             forwardingServer.shutdown()
             forwardingServer.server_close()
-            self._uio.info("Shutdown ssh port forwarding on %s." % (str(forwardingServer.server_address)))
+            self._info("Shutdown ssh port forwarding on %s." % (str(forwardingServer.server_address)))
 
     def startRevSSHTunnel(self, serverPort, destHost, destPort):
         """@brief Start an ssh reverse port forwarding tunnel
@@ -607,7 +729,7 @@ class SSHTunnelManager(object):
                              end of the ssh connection.
            @param destPort   The host TCP port of the tunnel destination at the local
                              end of the ssh connection."""
-        self._uio.info("Forwarding (reverse) Remote TCP server port (%d) to %s:%d on this end of the ssh connection." % (
+        self._info("Forwarding (reverse) Remote TCP server port (%d) to %s:%d on this end of the ssh connection." % (
         serverPort, destHost, destPort))
         # We add the None refs as the placeholders will be used later
         chan = None
@@ -632,7 +754,7 @@ class SSHTunnelManager(object):
             if sock:
                 sock.close()
 
-            self._uio.info("Shutdown reverse ssh port forwarding connection using remote server port %d." % (serverPort))
+            self._info("Shutdown reverse ssh port forwarding connection using remote server port %d." % (serverPort))
 
     def stopAllRevSSHTunnels(self):
         """@brief Stop all previously started reverse ssh port forwarding servers."""
@@ -647,7 +769,7 @@ class SSHTunnelManager(object):
             if sock:
                 sock.close()
 
-            self._uio.info("Shutdown reverse ssh port forwarding connection using remote server port %d." % (key))
+            self._info("Shutdown reverse ssh port forwarding connection using remote server port %d." % (key))
 
     def stopAllSSHTunnels(self):
         """@brief Stop all ssh tunnels."""
@@ -687,10 +809,10 @@ class SSHTunnelManager(object):
         try:
             sock.connect((destHost, destPort))
         except Exception as e:
-            self._uio.error('Forwarding (reverse) request to %s:%d failed: %r' % (destHost, destPort, e))
+            self._error('Forwarding (reverse) request to %s:%d failed: %r' % (destHost, destPort, e))
             return
 
-        self._uio.info('Connected!  Reverse tunnel open %r -> %r -> %r' % (chan.origin_addr,
+        self._info('Connected!  Reverse tunnel open %r -> %r -> %r' % (chan.origin_addr,
                                                                           chan.getpeername(), (destHost, destPort)))
         while True:
             r, w, x = select.select([sock, chan], [], [])
@@ -712,7 +834,7 @@ class SSHTunnelManager(object):
                     break
         chan.close()
         sock.close()
-        self._uio.info('Tunnel closed from server port %d' % (serverPort))
+        self._info('Tunnel closed from server port %d' % (serverPort))
 
 
 class ForwardingServer(socketserver.ThreadingTCPServer):
@@ -724,22 +846,34 @@ class ForwardingServer(socketserver.ThreadingTCPServer):
 class ForwardingHandler(socketserver.BaseRequestHandler):
     """@brief handler for ssh port forwarding connections."""
 
+    def _info(self, text):
+        """@brief Present an info level message to the user.
+           @param text The text to be presented to the user."""
+        if self.uo:
+            self.uo.info(text)
+
+    def _error(self, text):
+        """@brief Present an error level message to the user.
+           @param text The text to be presented to the user."""
+        if self.uo:
+            self.uo.error(text)
+
     def handle(self):
         try:
             chan = self.ssh_transport.open_channel('direct-tcpip',
                                                    (self.chain_host, self.chain_port),
                                                    self.request.getpeername())
         except Exception as e:
-            self.uo.error('Incoming request to %s:%d failed: %s' % (self.chain_host,
+            self._error('Incoming request to %s:%d failed: %s' % (self.chain_host,
                                                                     self.chain_port,
                                                                     repr(e)))
             return
         if chan is None:
-            self.uo.error('Incoming request to %s:%d was rejected by the SSH server.' %
+            self._error('Incoming request to %s:%d was rejected by the SSH server.' %
                           (self.chain_host, self.chain_port))
             return
 
-        self.uo.info('Connected!  Tunnel open %r -> %r -> %r' % (self.request.getpeername(),
+        self._info('Connected!  Tunnel open %r -> %r -> %r' % (self.request.getpeername(),
                                                                  chan.getpeername(),
                                                                  (self.chain_host, self.chain_port)))
         while True:
@@ -758,4 +892,4 @@ class ForwardingHandler(socketserver.BaseRequestHandler):
         peername = self.request.getpeername()
         chan.close()
         self.request.close()
-        self.uo.info('Tunnel closed from %r' % (peername,))
+        self._info('Tunnel closed from %r' % (peername,))
